"""Django models for ChewyAttachment"""

import uuid

from django.conf import settings
from django.db import models

from ..core.schemas import FileMetadata, UserContext


def get_storage_root():
    """Get storage root from Django settings"""
    from pathlib import Path

    chewy_settings = getattr(settings, "CHEWY_ATTACHMENT", {})
    if "STORAGE_ROOT" in chewy_settings:
        return chewy_settings["STORAGE_ROOT"]

    base_dir = getattr(settings, "BASE_DIR", Path.cwd())
    return Path(base_dir) / "media" / "attachments"


def get_attachment_table_name():
    """Get custom table name from Django settings"""
    chewy_settings = getattr(settings, "CHEWY_ATTACHMENT", {})
    return chewy_settings.get("TABLE_NAME", "chewy_attachment_files")


class Attachment(models.Model):
    """Attachment model for storing file metadata"""

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    original_name = models.CharField(max_length=255)
    storage_path = models.CharField(max_length=500)
    mime_type = models.CharField(max_length=100)
    size = models.BigIntegerField()
    owner_id = models.CharField(max_length=100, db_index=True)
    is_public = models.BooleanField(default=False, db_index=True)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)

    class Meta:
        # Use db_table property to support dynamic table name
        ordering = ["-created_at"]
        indexes = [
            models.Index(fields=["owner_id", "created_at"]),
        ]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Dynamically set table name
        if not hasattr(self._meta, '_db_table_set'):
            self._meta.db_table = get_attachment_table_name()
            self._meta._db_table_set = True

    def __str__(self):
        return f"{self.original_name} ({self.id})"

    def to_file_metadata(self) -> FileMetadata:
        """Convert to FileMetadata for permission checking"""
        return FileMetadata(
            id=str(self.id),
            original_name=self.original_name,
            storage_path=self.storage_path,
            mime_type=self.mime_type,
            size=self.size,
            owner_id=self.owner_id,
            is_public=self.is_public,
            created_at=self.created_at,
        )

    @staticmethod
    def get_user_context(request) -> UserContext:
        """Extract UserContext from Django request"""
        if hasattr(request, "user") and request.user.is_authenticated:
            user_id = str(request.user.id)
            return UserContext.authenticated(user_id)
        return UserContext.anonymous()



# Generated by Django 6.0.1 on 2026-01-13 10:12

import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('original_name', models.CharField(max_length=255)),
                ('storage_path', models.CharField(max_length=500)),
                ('mime_type', models.CharField(max_length=100)),
                ('size', models.BigIntegerField()),
                ('owner_id', models.CharField(db_index=True, max_length=100)),
                ('is_public', models.BooleanField(db_index=True, default=False)),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True)),
            ],
            options={
                'db_table': 'chewy_attachment_files',
                'ordering': ['-created_at'],
                'indexes': [models.Index(fields=['owner_id', 'created_at'], name='chewy_attac_owner_i_9ee24d_idx')],
            },
        ),
    ]



"""Django admin configuration for ChewyAttachment"""

from django.contrib import admin

from .models import Attachment


@admin.register(Attachment)
class AttachmentAdmin(admin.ModelAdmin):
    """Admin configuration for Attachment model"""

    list_display = ["id", "original_name", "mime_type", "size", "owner_id", "is_public", "created_at"]
    list_filter = ["is_public", "mime_type", "created_at"]
    search_fields = ["original_name", "owner_id"]
    readonly_fields = ["id", "storage_path", "mime_type", "size", "created_at"]
    ordering = ["-created_at"]



"""Pydantic schemas for ChewyAttachment FastAPI app"""

from datetime import datetime

from pydantic import BaseModel


class AttachmentResponse(BaseModel):
    """Response schema for attachment"""

    id: str
    original_name: str
    mime_type: str
    size: int
    owner_id: str
    is_public: bool
    created_at: datetime

    model_config = {"from_attributes": True}


class AttachmentUploadForm(BaseModel):
    """Form data for file upload (excluding file itself)"""

    is_public: bool = False


class ErrorResponse(BaseModel):
    """Error response schema"""

    detail: str



"""DRF serializers for ChewyAttachment"""

from rest_framework import serializers

from .models import Attachment


class AttachmentSerializer(serializers.ModelSerializer):
    """Serializer for Attachment model (read operations)"""

    class Meta:
        model = Attachment
        fields = [
            "id",
            "original_name",
            "mime_type",
            "size",
            "owner_id",
            "is_public",
            "created_at",
        ]
        read_only_fields = fields


class AttachmentUploadSerializer(serializers.Serializer):
    """Serializer for file upload"""

    file = serializers.FileField(required=True)
    is_public = serializers.BooleanField(default=False, required=False)

    def validate_file(self, value):
        """Validate uploaded file"""
        if not value:
            raise serializers.ValidationError("No file provided")
        return value



"""FastAPI router for ChewyAttachment"""

from fastapi import APIRouter, Depends, File, Form, HTTPException, UploadFile, status
from fastapi.responses import FileResponse
from sqlmodel import Session

from ..core.schemas import UserContext
from ..core.storage import FileStorageEngine
from . import crud
from .dependencies import (
    get_current_user_required,
    get_session,
    get_storage_engine,
    require_delete_permission,
    require_view_permission,
)
from .models import Attachment, AttachmentCreate
from .schemas import AttachmentResponse, ErrorResponse

router = APIRouter(prefix="/files", tags=["attachments"])


@router.post(
    "",
    response_model=AttachmentResponse,
    status_code=status.HTTP_201_CREATED,
    responses={
        401: {"model": ErrorResponse, "description": "Authentication required"},
    },
)
async def upload_file(
    file: UploadFile = File(...),
    is_public: bool = Form(default=False),
    session: Session = Depends(get_session),
    storage: FileStorageEngine = Depends(get_storage_engine),
    user: UserContext = Depends(get_current_user_required),
):
    """
    Upload a new file.

    - **file**: File to upload
    - **is_public**: Whether the file should be publicly accessible
    """
    content = await file.read()
    original_name = file.filename or "unnamed"

    result = storage.save_file(content, original_name)

    attachment_data = AttachmentCreate(
        original_name=original_name,
        storage_path=result.storage_path,
        mime_type=result.mime_type,
        size=result.size,
        owner_id=user.user_id,
        is_public=is_public,
    )

    attachment = crud.create_attachment(session, attachment_data)
    return attachment


@router.get(
    "/{attachment_id}",
    response_model=AttachmentResponse,
    responses={
        403: {"model": ErrorResponse, "description": "Permission denied"},
        404: {"model": ErrorResponse, "description": "Attachment not found"},
    },
)
async def get_file_info(
    attachment: Attachment = Depends(require_view_permission),
):
    """
    Get file metadata.

    - **attachment_id**: UUID of the attachment
    """
    return attachment


@router.get(
    "/{attachment_id}/content",
    responses={
        403: {"model": ErrorResponse, "description": "Permission denied"},
        404: {"model": ErrorResponse, "description": "Attachment not found"},
    },
)
async def download_file(
    attachment: Attachment = Depends(require_view_permission),
    storage: FileStorageEngine = Depends(get_storage_engine),
):
    """
    Download file content.

    - **attachment_id**: UUID of the attachment
    """
    try:
        file_path = storage.get_file_path(attachment.storage_path)
    except Exception:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="File not found on storage",
        )

    return FileResponse(
        path=file_path,
        media_type=attachment.mime_type,
        filename=attachment.original_name,
    )


@router.delete(
    "/{attachment_id}",
    status_code=status.HTTP_204_NO_CONTENT,
    responses={
        403: {"model": ErrorResponse, "description": "Permission denied"},
        404: {"model": ErrorResponse, "description": "Attachment not found"},
    },
)
async def delete_file(
    attachment: Attachment = Depends(require_delete_permission),
    session: Session = Depends(get_session),
    storage: FileStorageEngine = Depends(get_storage_engine),
):
    """
    Delete a file.

    Only the file owner can delete the file.

    - **attachment_id**: UUID of the attachment
    """
    storage.delete_file(attachment.storage_path)
    crud.delete_attachment(session, attachment)
    return None
